library ieee;
	use ieee.std_logic_1164.all;
	use ieee.numeric_std.all;
library work;
package project_component_package is

component sdram_pll is
	port (
		refclk   : in  std_logic := '0'; --  refclk.clk
		rst      : in  std_logic := '0'; --   reset.reset
		outclk_0 : out std_logic;        -- outclk0.clk
		locked   : out std_logic         --  locked.export
	);
end component;

type array_of_integer is array(natural range <>) of integer;
type array_of_std_logic_vector is array(natural range <>) of std_logic_vector;

component BrianHG_DDR3_CONTROLLER_v16_top is
	generic(
		FPGA_VENDOR :string := "Altera";
		FPGA_FAMILY :string := "Arria V GZ";
		BHG_OPTIMIZE_SPEED :std_logic_vector(0 downto 0) := "1";
		BHG_EXTRA_SPEED :std_logic_vector(0 downto 0) := "1";
		CLK_KHZ_IN :integer := 50000;
		CLK_IN_MULT :integer := 32;
		CLK_IN_DIV :integer := 4;
		DDR_TRICK_MTPS_CAP :integer := 0;
		INTERFACE_SPEED :string := "Half";
		DDR3_CK_MHZ :integer := 400; -- ((CLK_KHZ_IN*CLK_IN_MULT/CLK_IN_DIV)/1000);
		DDR3_SPEED_GRADE :string := "-107";
		DDR3_SIZE_GB :integer := 4;
		DDR3_WIDTH_DQ :integer := 8;
		DDR3_NUM_CHIPS :integer := 1;
		DDR3_NUM_CK :integer := 1;
		DDR3_WIDTH_ADDR :integer := 16;
		DDR3_WIDTH_BANK :integer := 3;
		DDR3_WIDTH_CAS :integer := 10;
		DDR3_WIDTH_DM :integer := 1; --(DDR3_WIDTH_DQ*DDR3_NUM_CHIPS/8);
		DDR3_WIDTH_DQS :integer := 1; --(DDR3_WIDTH_DQ*DDR3_NUM_CHIPS/8);
		DDR3_RWDQ_BITS :integer := 64; --(DDR3_WIDTH_DQ*DDR3_NUM_CHIPS*8);
		DDR3_ODT_RTT :integer := 40;
		DDR3_RZQ :integer := 40;
		DDR3_TEMP :integer := 85;
		DDR3_WDQ_PHASE :integer := 270;
		DDR3_RDQ_PHASE :integer := 0;
		DDR3_MAX_REF_QUEUE :std_logic_vector(4 downto 0) := "01000"; -- 8
		IDLE_TIME_uSx10 :std_logic_vector(6 downto 0) := "0000010"; --2
		SKIP_PUP_TIMER :std_logic_vector(0 downto 0) := "0";
		BANK_ROW_ORDER :string := "ROW_BANK_COL";
		PORT_ADDR_SIZE :integer := 29;--(DDR3_WIDTH_ADDR + DDR3_WIDTH_BANK + DDR3_WIDTH_CAS + (DDR3_WIDTH_DM-1));
		PORT_TOTAL :integer := 1;
		PORT_MLAYER_WIDTH :array_of_integer(0 to 3) := (2,2,2,2);
		PORT_VECTOR_SIZE :integer := 8;
		READ_ID_SIZE :integer := 4;
		DDR3_VECTOR_SIZE :integer := 5; --READ_ID_SIZE + 1;
		PORT_CACHE_BITS :integer := 64; --(8*DDR3_WIDTH_DM*8);
		CACHE_ADDR_WIDTH :integer := 3; --ceil(log2(real(PORT_CACHE_BITS/8)));-- $clog2(PORT_CACHE_BITS/8);
		PAGE_INDEX_BITS :integer := 10; --(DDR3_WIDTH_CAS + (DDR3_WIDTH_DM-1));
		USE_TOGGLE_OUTPUTS :std_logic_vector(0 downto 0) := "1";
		PORT_TOGGLE_INPUT: array_of_std_logic_vector(0 to 15)(0 downto 0) := ("0","0","0","0","0","0","0","0","0","0","0","0","0","0","0","0");
		
		PORT_R_DATA_WIDTH: array_of_std_logic_vector(0 to 15)(8 downto 0) := ("010000000","010000000","010000000","010000000","010000000","010000000","010000000","010000000","010000000","010000000","010000000","010000000","010000000","010000000","010000000","010000000");
		PORT_W_DATA_WIDTH: array_of_std_logic_vector(0 to 15)(8 downto 0) := ("010000000","010000000","010000000","010000000","010000000","010000000","010000000","010000000","010000000","010000000","010000000","010000000","010000000","010000000","010000000","010000000");
		
		PORT_PRIORITY: array_of_std_logic_vector(0 to 15)(1 downto 0) := ("00",  "00",  "00",  "00",  "00",  "00",  "00",  "00",  "00",  "00",  "00",  "00",  "00",  "00",  "00",  "00");
		PORT_READ_STACK: array_of_integer(0 to 15) := (24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24);
		PORT_W_CACHE_TOUT: array_of_std_logic_vector(0 to 15)(8 downto 0) := ("100000000","100000000","100000000","100000000","100000000","100000000","100000000","100000000","100000000","100000000","100000000","100000000","100000000","100000000","100000000","100000000");
		PORT_R_CACHE_TOUT_ENA: array_of_std_logic_vector(0 to 15)(0 downto 0) := ("1","1","1","1","1","1","1","1","1","1","1","1","1","1","1","1");
		PORT_R_CACHE_TOUT: array_of_std_logic_vector(0 to 15)(8 downto 0) := ("100000000","100000000","100000000","100000000","100000000","100000000","100000000","100000000","100000000","100000000","100000000","100000000","100000000","100000000","100000000","100000000");
		PORT_R_WDT_ENA: array_of_std_logic_vector(0 to 15)(0 downto 0) := ("1","1","1","1","1","1","1","1","1","1","1","1","1","1","1","1");
		PORT_CACHE_SMART: array_of_std_logic_vector(0 to 15)(0 downto 0) := ("1","1","1","1","1","1","1","1","1","1","1","1","1","1","1","1");
		PORT_DREG_READ: array_of_std_logic_vector(0 to 15)(0 downto 0) := ("1","1","1","1","1","1","1","1","1","1","1","1","1","1","1","1");
		PORT_MAX_BURST: array_of_std_logic_vector(0 to 15)(8 downto 0) := ("100000000","100000000","100000000","100000000","100000000","100000000","100000000","100000000","100000000","100000000","100000000","100000000","100000000","100000000","100000000","100000000");
		SMART_BANK :std_logic_vector(0 downto 0) := "0"
	);
	port (
		RST_IN :in std_logic;
		CLK_IN :in std_logic;                    -- External crystal clock which feeds the dedicated PLL for the DDR3.

		-- ****************************************
		-- DDR3 controller command clock and status
		-- ****************************************
		DDR3_CLK :out std_logic;
		DDR3_CLK_50 :out std_logic;
		DDR3_CLK_25 :out std_logic;
		CMD_CLK :out std_logic;                    -- DDR3 controller interface clock.  May be used by your entire FPGA
		RST_OUT :out std_logic;                    -- Stays high for at least 100ns after the PLL lock has been achieved.


		DDR3_READY :out std_logic;                 -- Goes high after the DDR3 has been initialized and successfully setup.
		SEQ_CAL_PASS :out std_logic;               -- Goes high if the DDR3 read calibration passes.
		PLL_LOCKED :out std_logic;                 -- Goes when the PLL is locked.
		RDCAL_data :out std_logic_vector(7 downto 0);                 -- A record of the PLL valid tuning positions.

		-- ****************************************
		-- DDR3 controller interface.
		-- ****************************************
		CMD_busy :out std_logic; 													 -- For each port, when high, the DDR3 controller will not accept an incoming command on that port.

		CMD_ena :in std_logic;  													-- Send a command.
		CMD_write_ena :in std_logic;  												-- Set high when you want to write data, low when you want to read data.

		CMD_addr :in std_logic_vector(PORT_ADDR_SIZE-1 downto 0);  					-- Command Address pointer.
		CMD_wdata :in std_logic_vector(PORT_CACHE_BITS-1 downto 0);  				-- During a 'CMD_write_req', this data will be written into the DDR3 at address 'CMD_addr'.
																					-- Each port's 'PORT_DATA_WIDTH' setting will prune the unused write data bits.
																					-- *** All channels of the 'CMD_wdata' will always be PORT_CACHE_BITS wide, however,
																					-- only the bottom 'PORT_W_DATA_WIDTH' bits will be active.

		CMD_wmask :in std_logic_vector(PORT_CACHE_BITS/8-1 downto 0);  				-- Write enable byte mask for the individual bytes within the 256 bit data bus.
																					-- When low, the associated byte will not be written.
																					-- Each port's 'PORT_DATA_WIDTH' setting will prune the unused mask bits.
																					-- *** All channels of the 'CMD_wmask' will always be 'PORT_CACHE_BITS/8' wide, however,
																					-- only the bottom 'PORT_W_DATA_WIDTH/8' bits will be active.

		CMD_read_vector_in :in std_logic_vector(PORT_VECTOR_SIZE-1 downto 0);  		-- The contents of the 'CMD_read_vector_in' during a read req will be sent to the
																					-- 'CMD_read_vector_out' in parallel with the 'CMD_read_data' during the 'CMD_read_ready' pulse.
																					-- *** All channels of the 'CMD_read_vector_in' will always be 'PORT_VECTOR_SIZE' wide,
																					-- it is up to the user to '0' the unused input bits on each individual channel.

		CMD_read_ready :out std_logic; 												-- Goes high for 1 clock when the read command data is valid.
		CMD_read_data :out std_logic_vector(PORT_CACHE_BITS-1 downto 0);  			-- Valid read data when 'CMD_read_ready' is high.
																					-- *** All channels of the 'CMD_read_data will' always be 'PORT_CACHE_BITS' wide, however,
																					-- only the bottom 'PORT_R_DATA_WIDTH' bits will be active.

		CMD_read_vector_out :out std_logic_vector(PORT_VECTOR_SIZE-1 downto 0);  	-- Returns the 'CMD_read_vector_in' which was sampled during the 'CMD_read_req' in parallel
																					-- with the 'CMD_read_data'.  This allows for multiple post reads where the output
																					-- has a destination pointer.

		CMD_priority_boost :in std_logic;  											-- Boosts the port's 'PORT_PRIORITY' parameter by a weight of 4 when set.

		SEQ_refresh_hold: in std_logic;                    							-- While high, no new refresh commands will be permitted.
																					-- ***** WARNING, use wisely, or the DDR3 contents will be corrupted.

		-- ****************************************      
		-- DDR3 Memory IO port                           
		-- ****************************************      
		DDR3_RESET_n :out std_logic;  -- DDR3 RESET# input pin.
		DDR3_CK_p :out std_logic_vector(DDR3_NUM_CK-1 downto 0);     -- DDR3_CLK ****************** YOU SHOULD SET THIS IO TO A DIFFERENTIAL LVDS or LVDS_E_3R
		DDR3_CK_n :out std_logic_vector(DDR3_NUM_CK-1 downto 0);     -- DDR3_CLK ****************** YOU SHOULD SET THIS IO TO A DIFFERENTIAL LVDS or LVDS_E_3R
														  -- ************************** port to generate the negative DDR3_CLK# output.
														  -- ************************** Generate an additional DDR_CK_p pair for every DDR3 ram chip. 

		DDR3_CKE :out std_logic;      -- DDR3 CKE

		DDR3_CS_n :out std_logic;     -- DDR3 CS#
		DDR3_RAS_n :out std_logic;    -- DDR3 RAS#
		DDR3_CAS_n :out std_logic;    -- DDR3 CAS#
		DDR3_WE_n :out std_logic;     -- DDR3 WE#
		DDR3_ODT :out std_logic;      -- DDR3 ODT

		DDR3_A :out std_logic_vector(DDR3_WIDTH_ADDR-1 downto 0);       -- DDR3 multiplexed address input bus
		DDR3_BA :out std_logic_vector(DDR3_WIDTH_BANK-1 downto 0);       -- DDR3 Bank select

		DDR3_DM :out std_logic_vector(DDR3_WIDTH_DM-1 downto 0);       -- DDR3 Write data mask. DDR3_DM[0] drives write DQ[7:0], DDR3_DM[1] drives write DQ[15:8]...
														  -- ***on x8 devices, the TDQS is not used and should be connected to GND or an IO set to GND.

		DDR3_DQ :out std_logic_vector(DDR3_WIDTH_DQ-1 downto 0);       -- DDR3 DQ data IO bus.
		DDR3_DQS_p :out std_logic_vector(DDR3_WIDTH_DQS-1 downto 0);    -- DDR3 DQS ********* IOs. DQS[0] drives DQ[7:0], DQS[1] drives DQ[15:8], DQS[2] drives DQ[23:16]...
		DDR3_DQS_n :out std_logic_vector(DDR3_WIDTH_DQS-1 downto 0);    -- DDR3 DQS ********* IOs. DQS[0] drives DQ[7:0], DQS[1] drives DQ[15:8], DQS[2] drives DQ[23:16]...
														  -- ****************** YOU SHOULD SET THIS IO TO A DIFFERENTIAL LVDS dedicated DQS pins for
														  -- ****************** proper parameter ' DDR3_RDQ_CLK_SEL = "DDR3_extDQS" ' support.

		-- ****************************************      
		-- Debug port...                        
		-- ****************************************      

		reset_phy :in std_logic;
		reset_cmd :in std_logic;
		

		-- **************************************************************************************
		-- This Write Data TAP port passes a copy of all the writes going to the DDR3 memory.
		-- This will allow to 'shadow' selected write addresses to other peripherals
		-- which may be accessed by all the multiple write ports.
		-- This port is synchronous to the CMD_CLK.
		-- **************************************************************************************
		TAP_WRITE_ENA :out std_logic;
		TAP_ADDR :out std_logic_vector(PORT_ADDR_SIZE-1 downto 0);
		TAP_WDATA :out std_logic_vector(PORT_CACHE_BITS-1 downto 0);
		TAP_WMASK :out std_logic_vector(PORT_CACHE_BITS/8-1 downto 0)
	);
end component;


end project_component_package;