library ieee;
	use ieee.std_logic_1164.all;
	use ieee.numeric_std.all;
	
library work;
	use work.project_component_package.all;
	
entity top is
port(
	i_reset			:in std_logic;
	i_clk_27		:in std_logic;
	i_clk_25		:in std_logic;
	i_clk_50		:in std_logic;
	
	i_video_data_wr	:in std_logic_vector(23 downto 0);
	i_video_addr_wr	:in std_logic_vector(19 downto 0);
	i_wr_en			:in std_logic;
	
	TestPoint_i_BANK8D	:out std_logic; -- XA13
	TestPoint_i_BANK7D	:out std_logic; -- XA9
	TestPoint_i_BANK7C	:out std_logic; -- XA8
	
	mem_a                     : out   std_logic_vector(15 downto 0);
	mem_ba                    : out   std_logic_vector(2 downto 0);
	mem_ck                    : out   std_logic;
	mem_ck_n                  : out   std_logic;
	mem_cke                   : out   std_logic;
	mem_cs_n                  : out   std_logic;
	mem_dm                    : out   std_logic;
	mem_ras_n                 : out   std_logic;
	mem_cas_n                 : out   std_logic;
	mem_we_n                  : out   std_logic;
	mem_reset_n               : out   std_logic;
	mem_dq                    : inout std_logic_vector(7 downto 0) := (others => '0');
	mem_dqs                   : inout std_logic;
	mem_dqs_n                 : inout std_logic;
	mem_odt                   : out   std_logic;
	oct_rzqin                 : in    std_logic;
	
	o_lan_rst		:out std_logic_vector(1 downto 0);
	o_tmds_p		:out std_logic_vector(2 downto 0);
	o_tmds_clk_p	:out std_logic
);
end top;

architecture arch of top is

	constant data_bus_width	:integer := 8;
	constant addr_bus_width :integer := 29;
	constant avl_data_bus_width :integer := 32;
	constant avl_be_width :integer := 4;

	signal px_clk			:std_logic;
	signal tmds_clk			:std_logic;
	signal vdata_clk		:std_logic;
	signal sdram_clk		:std_logic;

	signal vdata_rd			:std_logic_vector(23 downto 0);
	signal vdata_addr_rd	:std_logic_vector(19 downto 0);
	
	signal sdram_addr		:std_logic_vector(addr_bus_width - 1 downto 0);
	signal sdram_data_wr	:std_logic_vector(data_bus_width - 1 downto 0);
	signal sdram_write_en	:std_logic := '0';
	signal sdram_data_rd	:std_logic_vector(data_bus_width - 1 downto 0);
	signal sdram_read_en	:std_logic := '0';
	signal sdram_ready		:std_logic := '0';
	signal sdram_read_ready	:std_logic := '0';

	type state_type is (s0, s1, s2, s3);
	signal state 			:state_type := s0;
	
begin
	o_tmds_clk_p <= px_clk;
	
	sdram_pll_i: sdram_pll
	port map(
		rst      => not i_reset,
		refclk   => i_clk_27,
		outclk_0 => sdram_clk
	);	
		
	sdram_i: BrianHG_DDR3_CONTROLLER_v16_top 
	port map(
		RST_IN => '0',
		CLK_IN => sdram_clk,
		CMD_ena => '0',
		CMD_write_ena => '0',
		CMD_priority_boost => '0',
		SEQ_refresh_hold => '0',
		reset_phy => '0',
		reset_cmd => '0',
		
		CMD_addr => sdram_addr,
		CMD_wdata => x"00000000000000" & sdram_data_wr,
		
		CMD_read_ready => sdram_read_ready,
		CMD_read_data(7 downto 0) => sdram_data_rd,
		
		CMD_wmask => x"ff",
		CMD_read_vector_in => x"ff",

		DDR3_A			=> mem_a,		
		DDR3_BA			=> mem_ba,
		DDR3_CK_p(0)	=> mem_ck,
		DDR3_CK_n(0)	=> mem_ck_n,
		DDR3_CKE		=> mem_cke,
		DDR3_CS_n		=> mem_cs_n,
		DDR3_DM(0)		=> mem_dm,
		DDR3_RAS_n		=> mem_ras_n,
		DDR3_CAS_n		=> mem_cas_n,
		DDR3_WE_n		=> mem_we_n,
		DDR3_RESET_n	=> mem_reset_n,
		DDR3_DQ			=> mem_dq,
		DDR3_DQS_p(0)	=> mem_dqs,
		DDR3_DQS_n(0)	=> mem_dqs_n,
		DDR3_ODT		=> mem_odt
	);

end arch;